part of client_filtering;

class FilterCriteria<TValue extends dynamic> with IJsonable {
  final String fieldName;
  final Logic op;
  final Operators logicalOperator;
  final TValue? value;
  final TValue? value2;

  const FilterCriteria({
    required this.fieldName,
    required this.op,
    required this.value,
    this.value2,
    required this.logicalOperator,
  });

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is FilterCriteria &&
        other.fieldName == fieldName &&
        other.op == op &&
        other.logicalOperator == logicalOperator &&
        other.value == value &&
        other.value2 == value2;
  }

  @override
  int get hashCode {
    return fieldName.hashCode ^
        op.hashCode ^
        logicalOperator.hashCode ^
        value.hashCode ^
        value2.hashCode;
  }

  FilterCriteria copyWith({
    String Function()? fieldName,
    Logic Function()? op,
    Operators Function()? logicalOperator,
    TValue? Function()? value,
    TValue? Function()? value2,
  }) {
    return FilterCriteria(
      fieldName: fieldName == null ? this.fieldName : fieldName(),
      op: op == null ? this.op : op(),
      logicalOperator:
          logicalOperator == null ? this.logicalOperator : logicalOperator(),
      value: value == null ? this.value : value(),
      value2: value2 == null ? this.value2 : value2(),
    );
  }

  @override
  String toString() {
    return 'FilterCriteria(fieldName: $fieldName, op: $op, logicalOperator: $logicalOperator, value: $value), value2: $value2)';
  }

  Map<String, dynamic> toJson() {
    return {
      'fieldName': fieldName,
      'op': serializeEnumString(op.toString()),
      'logicalOperator': serializeEnumString(logicalOperator.toString()),
      'value': _valueToString(value),
      'value2': _valueToString(value2),
    };
  }

  factory FilterCriteria.fromJson(Map<String, dynamic> map) {
    return FilterCriteria(
      fieldName: map['fieldName'],
      op: deseralizeEnumString(map['op'], Logic.values),
      logicalOperator: deseralizeEnumString(
        map['logicalOperator'],
        Operators.values,
      ),
      value: _parseValue(map['value']),
      value2: _parseValue(map['value2']),
    );
  }

  static TValue? _parseValue<TValue extends dynamic>(String? value) {
    if (value == null) return null;

    switch (TValue) {
      case double:
        return double.parse(value) as TValue;
      case DateTime:
        return DateTime.parse(value) as TValue;
      case int:
        return int.parse(value) as TValue;
      case num:
        return num.parse(value) as TValue;
      case String:
        return value as TValue;
      case bool:
        return value == "True" || value == "true" || value == "1"
            ? true
            : false as TValue;
      default:
        if (TValue == TimeOfDay)
          return TimeOfDay.fromDateTime(DateTime.parse(value)) as TValue;
        throw UnsupportedError(
            "The type ${TValue.toString()} is not supported for deserialization.");
    }
  }

  static String? _valueToString<TValue extends dynamic>(TValue? value) {
    if (value == null) return null;

    switch (TValue) {
      case DateTime:
        return (value as DateTime).toIso8601String();
      case TimeOfDay:
        final tod = (value as TimeOfDay);
        return DateTime(1, 1, 1, tod.hour, tod.minute).toIso8601String();
      default:
        return value.toString();
    }
  }
}
